package com.gentics.portal.node.connector.examples;

import java.io.File;
import java.net.URI;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import com.gentics.api.lib.datasource.Datasource;
import com.gentics.api.lib.expressionparser.Expression;
import com.gentics.api.lib.expressionparser.filtergenerator.DatasourceFilter;
import com.gentics.api.lib.resolving.Resolvable;
import com.gentics.api.portalnode.connector.PLinkInformation;
import com.gentics.api.portalnode.connector.PLinkReplacer;
import com.gentics.api.portalnode.connector.PortalConnectorFactory;
import com.gentics.api.portalnode.connector.PortalConnectorHelper;

/**
 * A more advanced example, how to retrieve folders and their content. In
 * addition it demonstrates how to access differerent attribute types.<br>
 * <br>
 * 1 - Text (Short) - Used for names, filenames, etc.<br>
 * 2 - Link to another Object - Used to link pages to folders.<br>
 * 5 - Text (Long) - Used to store page content.<br>
 * 6 - Binary Data - Used to store image/file content.<br>
 */
public class AdvancedPortalConnectorExample {

    /**
     * @param args
     */
    public static void main(String[] args) {
        try {
            Datasource ds = getDatasource();

            // Retrieve all folders.
            Collection folders = getObjects(ds, "object.obj_type == 10002");
            for (Iterator folderIterator = folders.iterator(); folderIterator.hasNext();) {
                Resolvable folder = (Resolvable) folderIterator.next();
                System.out.println(folder.get("name"));

                // Retrieving all pages within this folder ...
                String folderId = (String) folder.get("contentid");
                Collection pages = getObjects(ds,
                        "object.obj_type == 10007 AND object.folder_id == " + folderId);
                for (Iterator pageIterator = pages.iterator(); pageIterator.hasNext();) {
                    Resolvable pageResolvable = (Resolvable) pageIterator.next();

                    // We got a page.

                    // Get Name of the Page (type Text Short, 1)
                    String name = (String) pageResolvable.get("name");
                    System.out.println("  Page Name: " + name);

                    // Get Page Content (type Text Long, 5)
                    String pageContent = (String) pageResolvable.get("content");
                    String fileName = (String) pageResolvable.get("filename");
                    System.out.println("    Page Content Length: " + pageContent.length()
                            + " Filename: " + fileName);

                    // Replace the <plink>s in this page.
                    pageContent = replacePLinks(pageContent);

                    // For demonstration only retrieve the folder.
                    // Since it is defined as a linked attribute a resolvable
                    // will be returned.
                    // (type Link to another Object, 2)
                    Resolvable folder_test = (Resolvable) pageResolvable.get("folder_id");
                    if (folder_test != null) {
                        // This resolvable can be accessed to retrieve the name
                        // of the folder.
                        String folderName = (String) folder_test.get("name");
                        System.out.println("     Folder: " + folderName);
                    }
                }

            }

            // Displaying all images ...
            Collection files = getObjects(ds, "object.obj_type == 10008"); // Get
                                                                            // all
                                                                            // files.
            for (Iterator fileIterator = files.iterator(); fileIterator.hasNext();) {
                Resolvable resolvable = (Resolvable) fileIterator.next();
                String mimetype = (String) resolvable.get("mimetype");

                // Since we iterate all files, test if this is really an image.
                if (mimetype.startsWith("image/")) {
                    String name = (String) resolvable.get("name");
                    System.out.println("Image: " + name);

                    // Get binary content of image. (Type Binary Data, 6)
                    byte[] binarycontent = (byte[]) resolvable.get("binarycontent");

                    // Make sure we got a binarycontent of the image.
                    if (binarycontent != null) {
                        System.out.println("  Image Size: " + binarycontent.length + " bytes.");
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Replaces &lt;plink&gt; tags in a given page content.
     * @param pageContent the content of a page.
     * @return the page content with replaced plinks.
     */
    protected static String replacePLinks(String pageContent) {
        // Using the PortalConnectorHelper to replace plinks.
        return PortalConnectorHelper.replacePLinks(pageContent, new PLinkReplacer() {
            public String replacePLink(PLinkInformation plink) {
                String contentid = plink.getContentId();
                // Now translate the contentid to a portal-
                // specific link.
                return "/MyPortalURL?contentid=" + contentid;
            }
        });
    }

    /**
     * Create the datasource
     * @return datasource
     * @throws Exception
     */
    protected static Datasource getDatasource() throws Exception {
        Properties handleProperties = new Properties();
        // Load the properties for the datasource connection from the
        // filesystem.
        handleProperties.load(AdvancedPortalConnectorExample.class
                .getResourceAsStream("jdbc-handle.properties"));
        // for usage of jndi datasources:
        // handleProperties.load(DatasourceAccess.class.getResourceAsStream("jndi-handle.properties"));
        
        // Now activate caching ..
        Map datasourceProperties = new HashMap();
        datasourceProperties.put("cache","true");
        
        // Configure Cache Regions. First find current path (to cache.ccf) and set system property accordingly
        String thisDirectory = new File(new URI(AdvancedPortalConnectorExample.class.getResource("cache.ccf").toString())).getParent();
        // Usually you would want this to be set through -Dcom.gentics.portal.node.confpath=... to a constant path.
        System.setProperty( "com.gentics.portalnode.confpath", thisDirectory );

        return PortalConnectorFactory.createDatasource(handleProperties, datasourceProperties);
    }

    /**
     * Get the objects filtered by the given rule
     * @param ds datasource
     * @param rule rule
     * @return collection of DatasourceRow objects
     * @throws Exception
     */
    protected static Collection getObjects(Datasource ds, String rule) throws Exception {
        // Create the RuleTree we use to search for the requested content objects.
        Expression expression = PortalConnectorFactory.createExpression(rule);
        DatasourceFilter filter = ds.createDatasourceFilter(expression);

        // Return the results.
        return ds.getResult(filter, new String[] { "name" }, 0, 10, null);
    }
}
